//example is based on sin wave generator example with addind FFT(Fast Fourier Transform) to see frequency domain instead time domain
//display data on PC with SerialPlot https://hackaday.io/project/5334-serialplot-realtime-plotting-software

#define led          GPIOC_ODR.B13
#define BUF_DEPTH    1024
#define SIN_STEPS    120
#define PI           3.1415926

//sin table generated by Smart Sine http://tahmidmc.blogspot.com/2012/10/smart-sine-software-to-generate-sine.html
const unsigned int sinTable[SIN_STEPS] =
{500, 526, 552, 578, 604, 629, 655, 679, 703, 727, 750, 772, 794, 815, 835, 854, 872, 889, 905, 919, 933, 946, 957, 967, 976, 983, 989, 994, 997, 999, 1000, 999, 997, 994, 989, 983, 976, 967, 957, 946, 933, 919, 905, 889, 872, 854, 835, 815, 794, 772, 750, 727, 703, 679, 655, 629, 604, 578, 552, 526, 500, 474, 448, 422, 396, 371, 345, 321, 297, 273, 250, 228, 206, 185, 165, 146, 128, 111, 95, 81, 67, 54, 43, 33, 24, 17, 11, 6, 3, 1, 0, 1, 3, 6, 11, 17, 24, 33, 43, 54, 67, 81, 95, 111, 128, 146, 165, 185, 206, 228, 250, 273, 297, 321, 345, 371, 396, 422, 448, 474};

unsigned int num=0;
char data_send_time=0;
float realNumbers_float[BUF_DEPTH];
float imaginaryNumbers_float[BUF_DEPTH];

extern int FFT( int dir,int m,float *x,float *y);// dir=1(forward transform),m=10(2^10=1024)

 void adc_inter() iv IVT_INT_ADC1_2 // adc interrupt after EOC
 {
     ADC1_SRbits.EOC=0;

     if(data_send_time==0)// change buffer only if it is not used
     {
         led=0;
         if(num<BUF_DEPTH)// collect buffer
         {
             realNumbers_float[num]=ADC1_DRbits.DATA;
             imaginaryNumbers_float[num]=0;// we only have real numbers from ADC
             num++;
         }
         else // buffer is full and time to send it to PC
         {
             num=0;
             data_send_time=1;
         }
         led=1;
     }
 }

void timer_3_init()// 1000Hz(1ms between samples)
 {
       RCC_APB1ENR.TIM3EN = 1;       // Enable clock gating for timer module
       TIM3_CR1.CEN = 0;             // Disable timer
       TIM3_PSC = 7030;              // You might need to trim timer's prescaler for more accuracy(i trimmed it here from 7200 to 7030 and reduce measurement error of 3%)
       TIM3_ARR = 9;                //
       TIM3_CR2bits.MMS=2;           // The update event is selected as trigger output (TRGO)
       TIM3_CR1.CEN = 1;             // Enable timer
 }

 void ADC_init(void)//ADC triggered by timer 3
 {
        GPIO_Analog_Input(&GPIOA_BASE, _GPIO_PINMASK_0);// channel 0

        RCC_APB2ENRbits.ADC1EN=1;       //enable clock on ADC1
        ADC1_CR2bits.EXTSEL=4;          //Timer 3 TRGO event
        ADC1_CR2bits.EXTTRIG=1;         //External trigger for ADC on
        ADC1_SMPR2bits.SMP0=1;          //Sample time 7.5 cycles
        ADC1_CR1bits.EOCIE=1;           //Interrupt enable for EOC
        ADC1_CR2bits.ADON=1;            //ADC enable
        NVIC_IntEnable(IVT_INT_ADC1_2); //Interrupt enable for ADC1_2
 }

 //sin_real_freq=FCPU/((TIM4_PSC+1)*(TIM4_ARR+1)*SIN_STEPS)
 //sin_real_freq(here)=72000000/((8+1)*(999+1)*120)=66,66666 Hz // in this example frequency set like that to not to fit in buffer whole number of periods and to we can see "bell-shaped" specrum, whitch can be emproved by windowing

 void init_TIM4()// init timer for PWM by table
{
        RCC_APB1ENRbits.TIM4EN=1;      //enable clock on timer 4
        TIM4_CCERbits.CC3E=1;          //CC4 channel configured as output
        TIM4_CCMR2_Outputbits.OC3M=6;  //Output compare 3 mode (PWM)
        TIM4_CCMR2_Outputbits.OC3PE=1; //Output compare 3 preload enable
        TIM4_DIERbits.CC3DE=1;         //Capture/Compare 3 DMA request enable
        TIM4_CR1bits.CEN=1;            //Counter enable
        TIM4_CR1bits.ARPE=1;           //Auto-reload preload enable
        TIM4_PSC=8;                   //Prescaler value
        TIM4_ARR=999;                 //max count
        TIM4_CCR3=500;                //CCR3 is the value to be loaded in the actual capture/compare 3 register (preload value).
}

void init_DMA()// init sending data from table to timer without CPU  5ch
{
        RCC_AHBENRbits.DMA1EN=1; //DMA1 clock enable
        DMA1_CPAR5=&TIM4_CCR3;   //DMA1 channel 5 peripheral address register(set address of TIM4_CCR3)
        DMA1_CMAR5=&sinTable;    //DMA1 channel x memory address register(set table address)
        DMA1_CNDTR5=SIN_STEPS;   //DMA channel x number of data register
        DMA1_CCR5=0x000005B1;    //EN=1(Channel enable),DIR=1(Data transfer direction:Read from memory),CIRC=1(Circular mode enabled),MINC=1(Memory increment mode enabled),PSIZE=MSIZE=01(16-bits)
}

void init_GPIO()// init gpio to work with timer
{
        RCC_APB2ENRbits.IOPBEN=1; //IO port B clock enable
        RCC_APB2ENRbits.AFIOEN=1; //Alternate function IO clock enable
        GPIOB_CRHbits.MODE8=1;    //10: Alternate function output Push-pull
        GPIOB_CRHbits.CNF8=2;     //Output mode, max speed 2 MHz
}

void init_PWM_sin()
{
        init_GPIO();// init gpio to work with timer
        init_TIM4();// init timer for PWM by table
        init_DMA(); // init sending data from table to timer without CPU  5ch
}

void init_peripherals(void)
{
     GPIO_Digital_Output(&GPIOC_BASE, _GPIO_PINMASK_13); //led
     GPIOC_ODR.B13=1;//off

     ADC_init();
     timer_3_init();

     init_PWM_sin();

     UART1_Init(115200);
}

void adc_buf_send(void)//send buf to PC
{
        unsigned int i=0;
        int tempi=0;
        char temp=0;

        for(i=0;i<BUF_DEPTH/2;i++)   // put data byte after byte
        {
            tempi=realNumbers_float[i];
            temp=tempi>>8;
            UART1_Write(temp);
            temp=tempi&0x00FF;
            UART1_Write(temp);
            //Delay_ms(1);
        }

}

void from_adc_to_milivolts(void)
{
        unsigned int i=0;
        unsigned long temp=0;
        for(i=0;i<BUF_DEPTH;i++)   //
        {
             temp=realNumbers_float[i];
             temp=temp*3335;//vcc in milivolts
             temp=temp/4095;
             realNumbers_float[i]=temp;
        }
}

void module_bufer(void)// calculate module of complex fft numbers
{
   int k=0;
   double max=0;
   //module of complex number = sqrt(Real*Real+Imaginary*Imaginary)
   for(k=0;k<BUF_DEPTH/2;k++)//only 1/2 of buffer will be usable after FFT, other half just mirrored
   {
        realNumbers_float[k]=sqrt(realNumbers_float[k]*realNumbers_float[k]+imaginaryNumbers_float[k]*imaginaryNumbers_float[k]);
        if(max<realNumbers_float[k])max=realNumbers_float[k];// find max module for dB transfer(if you want to most big harmonic will be base for others, 0dB)
   }
   //from milivolts to dB
   for(k=0;k<BUF_DEPTH/2;k++)
   {
        realNumbers_float[k]=(float)10*log10(realNumbers_float[k]/max);//you can replace "max" on 1, then the values of the array will be in relation to the unit of measurement of voltage (mV)
        if(realNumbers_float[k]<-100)realNumbers_float[k]=-100;// low signal limitation, to emprove clearness on PC view
   }
}

float w_hann( float pos)
{
        return (0.5*(1-cos(pos)));
}

float w_hamming( float pos)
{
        return (0.54 - 0.46 * cos(pos));
}

float w_blackman( float pos)
{
    return (0.42 - 0.5 * cos(pos) + 0.08 * cos(2.0*pos));
}

float w_blackman_harris(float pos)
{
    return (0.35875 - 0.48829 * cos(pos) + 0.14128 * cos(2.0*pos)- 0.01168 * cos(3.0*pos));
}

float w_nuttall(float pos)
{
    return (0.355768 - 0.487396 * cos(pos) + 0.144232 * cos(2.0*pos)- 0.012604 * cos(3.0*pos));
}

float flat_top_window(float pos)
{
    return (1.0 - 1.93 * cos(pos) + 1.29 * cos(2.0*pos)- 0.388 * cos(3.0*pos)+0.028 * cos(4.0*pos));
}

float w_blackman_nuttall(float pos)
{
    return (0.3635819 - 0.4891775 * cos(pos) + 0.1365995 * cos(2.0*pos)- 0.0106411 * cos(3.0*pos));
}

void do_window(float* pOut)//  measured array is equal to zero elsewhere out of window and will fit now without "bell-shape"
{
    unsigned int i=0;

    for (i=0; i<BUF_DEPTH; i++)//whole buffer wil be recalculated
    {
         pOut[i] *= w_blackman_nuttall((float)(2*PI*i)/(BUF_DEPTH-1));// you can choose any window function from above by your own taste and paste its name instead "w_blackman_nuttall"
    }
}

void main()//
{
      init_peripherals();

      while(1)
      {
           if(data_send_time==1)//
           {
               from_adc_to_milivolts();
               do_window(realNumbers_float);                       // you can disable windowing by commenting this line
               FFT(1,10,realNumbers_float,imaginaryNumbers_float); // you can disable FFT and see what windowing do with array by commenting this line and line below(module_bufer();).Note, that if you want to see all buffer(not half), correct "BUF_DEPTH/2" to "BUF_DEPTH" in "adc_buf_send" function
               module_bufer();                                     // prepare data to view
               adc_buf_send();                                     // send buf to PC
               data_send_time=0;                                   // start to collect new buffer
           }
      }
}